from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="DataProviderStatus")


@attr.s(auto_attribs=True)
class DataProviderStatus:
    """  """

    is_active: Union[Unset, bool] = UNSET
    driver: Union[Unset, str] = UNSET
    error: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_active = self.is_active
        driver = self.driver
        error = self.error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_active is not UNSET:
            field_dict["is_active"] = is_active
        if driver is not UNSET:
            field_dict["driver"] = driver
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_active = d.pop("is_active", UNSET)

        driver = d.pop("driver", UNSET)

        error = d.pop("error", UNSET)

        data_provider_status = cls(
            is_active=is_active,
            driver=driver,
            error=error,
        )

        data_provider_status.additional_properties = d
        return data_provider_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FTPPassivePortRange")


@attr.s(auto_attribs=True)
class FTPPassivePortRange:
    """  """

    start: Union[Unset, int] = UNSET
    end: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start = self.start
        end = self.end

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start = d.pop("start", UNSET)

        end = d.pop("end", UNSET)

        ftp_passive_port_range = cls(
            start=start,
            end=end,
        )

        ftp_passive_port_range.additional_properties = d
        return ftp_passive_port_range

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LoginMethods(str, Enum):
    PUBLICKEY = "publickey"
    PASSWORD = "password"
    KEYBOARD_INTERACTIVE = "keyboard-interactive"
    PUBLICKEYPASSWORD = "publickey+password"
    PUBLICKEYKEYBOARD_INTERACTIVE = "publickey+keyboard-interactive"
    TLSCERTIFICATE = "TLSCertificate"
    TLSCERTIFICATEPASSWORD = "TLSCertificate+password"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="S3Config")


@attr.s(auto_attribs=True)
class S3Config:
    """ S3 Compatible Object Storage configuration details """

    bucket: Union[Unset, str] = UNSET
    region: Union[Unset, str] = UNSET
    access_key: Union[Unset, str] = UNSET
    access_secret: Union[Unset, Secret] = UNSET
    endpoint: Union[Unset, str] = UNSET
    storage_class: Union[Unset, str] = UNSET
    upload_part_size: Union[Unset, int] = UNSET
    upload_concurrency: Union[Unset, int] = UNSET
    key_prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bucket = self.bucket
        region = self.region
        access_key = self.access_key
        access_secret: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.access_secret, Unset):
            access_secret = self.access_secret.to_dict()

        endpoint = self.endpoint
        storage_class = self.storage_class
        upload_part_size = self.upload_part_size
        upload_concurrency = self.upload_concurrency
        key_prefix = self.key_prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bucket is not UNSET:
            field_dict["bucket"] = bucket
        if region is not UNSET:
            field_dict["region"] = region
        if access_key is not UNSET:
            field_dict["access_key"] = access_key
        if access_secret is not UNSET:
            field_dict["access_secret"] = access_secret
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if storage_class is not UNSET:
            field_dict["storage_class"] = storage_class
        if upload_part_size is not UNSET:
            field_dict["upload_part_size"] = upload_part_size
        if upload_concurrency is not UNSET:
            field_dict["upload_concurrency"] = upload_concurrency
        if key_prefix is not UNSET:
            field_dict["key_prefix"] = key_prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bucket = d.pop("bucket", UNSET)

        region = d.pop("region", UNSET)

        access_key = d.pop("access_key", UNSET)

        access_secret: Union[Unset, Secret] = UNSET
        _access_secret = d.pop("access_secret", UNSET)
        if not isinstance(_access_secret, Unset):
            access_secret = Secret.from_dict(_access_secret)

        endpoint = d.pop("endpoint", UNSET)

        storage_class = d.pop("storage_class", UNSET)

        upload_part_size = d.pop("upload_part_size", UNSET)

        upload_concurrency = d.pop("upload_concurrency", UNSET)

        key_prefix = d.pop("key_prefix", UNSET)

        s3_config = cls(
            bucket=bucket,
            region=region,
            access_key=access_key,
            access_secret=access_secret,
            endpoint=endpoint,
            storage_class=storage_class,
            upload_part_size=upload_part_size,
            upload_concurrency=upload_concurrency,
            key_prefix=key_prefix,
        )

        s3_config.additional_properties = d
        return s3_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.azure_blob_fs_config_access_tier import AzureBlobFsConfigAccessTier
from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="AzureBlobFsConfig")


@attr.s(auto_attribs=True)
class AzureBlobFsConfig:
    """ Azure Blob Storage configuration details """

    container: Union[Unset, str] = UNSET
    account_name: Union[Unset, str] = UNSET
    account_key: Union[Unset, Secret] = UNSET
    sas_url: Union[Unset, str] = UNSET
    endpoint: Union[Unset, str] = UNSET
    upload_part_size: Union[Unset, int] = UNSET
    upload_concurrency: Union[Unset, int] = UNSET
    access_tier: Union[Unset, AzureBlobFsConfigAccessTier] = UNSET
    key_prefix: Union[Unset, str] = UNSET
    use_emulator: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        container = self.container
        account_name = self.account_name
        account_key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.account_key, Unset):
            account_key = self.account_key.to_dict()

        sas_url = self.sas_url
        endpoint = self.endpoint
        upload_part_size = self.upload_part_size
        upload_concurrency = self.upload_concurrency
        access_tier: Union[Unset, str] = UNSET
        if not isinstance(self.access_tier, Unset):
            access_tier = self.access_tier.value

        key_prefix = self.key_prefix
        use_emulator = self.use_emulator

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if container is not UNSET:
            field_dict["container"] = container
        if account_name is not UNSET:
            field_dict["account_name"] = account_name
        if account_key is not UNSET:
            field_dict["account_key"] = account_key
        if sas_url is not UNSET:
            field_dict["sas_url"] = sas_url
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if upload_part_size is not UNSET:
            field_dict["upload_part_size"] = upload_part_size
        if upload_concurrency is not UNSET:
            field_dict["upload_concurrency"] = upload_concurrency
        if access_tier is not UNSET:
            field_dict["access_tier"] = access_tier
        if key_prefix is not UNSET:
            field_dict["key_prefix"] = key_prefix
        if use_emulator is not UNSET:
            field_dict["use_emulator"] = use_emulator

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        container = d.pop("container", UNSET)

        account_name = d.pop("account_name", UNSET)

        account_key: Union[Unset, Secret] = UNSET
        _account_key = d.pop("account_key", UNSET)
        if not isinstance(_account_key, Unset):
            account_key = Secret.from_dict(_account_key)

        sas_url = d.pop("sas_url", UNSET)

        endpoint = d.pop("endpoint", UNSET)

        upload_part_size = d.pop("upload_part_size", UNSET)

        upload_concurrency = d.pop("upload_concurrency", UNSET)

        access_tier: Union[Unset, AzureBlobFsConfigAccessTier] = UNSET
        _access_tier = d.pop("access_tier", UNSET)
        if not isinstance(_access_tier, Unset):
            access_tier = AzureBlobFsConfigAccessTier(_access_tier)

        key_prefix = d.pop("key_prefix", UNSET)

        use_emulator = d.pop("use_emulator", UNSET)

        azure_blob_fs_config = cls(
            container=container,
            account_name=account_name,
            account_key=account_key,
            sas_url=sas_url,
            endpoint=endpoint,
            upload_part_size=upload_part_size,
            upload_concurrency=upload_concurrency,
            access_tier=access_tier,
            key_prefix=key_prefix,
            use_emulator=use_emulator,
        )

        azure_blob_fs_config.additional_properties = d
        return azure_blob_fs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SecretStatus(str, Enum):
    PLAIN = "Plain"
    AES_256_GCM = "AES-256-GCM"
    SECRETBOX = "Secretbox"
    GCP = "GCP"
    AWS = "AWS"
    VAULTTRANSIT = "VaultTransit"
    REDACTED = "Redacted"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="VirtualFolder")


@attr.s(auto_attribs=True)
class VirtualFolder:
    """ A virtual folder is a mapping between a SFTPGo virtual path and a filesystem path outside the user home directory. The specified paths must be absolute and the virtual path cannot be "/", it must be a sub directory. The parent directory for the specified virtual path must exist. SFTPGo will try to automatically create any missing parent directory for the configured virtual folders at user login. """

    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        virtual_folder = cls()

        virtual_folder.additional_properties = d
        return virtual_folder

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.gcs_config_automatic_credentials import GCSConfigAutomaticCredentials
from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="GCSConfig")


@attr.s(auto_attribs=True)
class GCSConfig:
    """ Google Cloud Storage configuration details. The "credentials" field must be populated only when adding/updating a user. It will be always omitted, since there are sensitive data, when you search/get users """

    bucket: Union[Unset, str] = UNSET
    credentials: Union[Unset, Secret] = UNSET
    automatic_credentials: Union[Unset, GCSConfigAutomaticCredentials] = UNSET
    storage_class: Union[Unset, str] = UNSET
    key_prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bucket = self.bucket
        credentials: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.credentials, Unset):
            credentials = self.credentials.to_dict()

        automatic_credentials: Union[Unset, int] = UNSET
        if not isinstance(self.automatic_credentials, Unset):
            automatic_credentials = self.automatic_credentials.value

        storage_class = self.storage_class
        key_prefix = self.key_prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bucket is not UNSET:
            field_dict["bucket"] = bucket
        if credentials is not UNSET:
            field_dict["credentials"] = credentials
        if automatic_credentials is not UNSET:
            field_dict["automatic_credentials"] = automatic_credentials
        if storage_class is not UNSET:
            field_dict["storage_class"] = storage_class
        if key_prefix is not UNSET:
            field_dict["key_prefix"] = key_prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bucket = d.pop("bucket", UNSET)

        credentials: Union[Unset, Secret] = UNSET
        _credentials = d.pop("credentials", UNSET)
        if not isinstance(_credentials, Unset):
            credentials = Secret.from_dict(_credentials)

        automatic_credentials: Union[Unset, GCSConfigAutomaticCredentials] = UNSET
        _automatic_credentials = d.pop("automatic_credentials", UNSET)
        if not isinstance(_automatic_credentials, Unset):
            automatic_credentials = GCSConfigAutomaticCredentials(
                _automatic_credentials
            )

        storage_class = d.pop("storage_class", UNSET)

        key_prefix = d.pop("key_prefix", UNSET)

        gcs_config = cls(
            bucket=bucket,
            credentials=credentials,
            automatic_credentials=automatic_credentials,
            storage_class=storage_class,
            key_prefix=key_prefix,
        )

        gcs_config.additional_properties = d
        return gcs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...models.backup_data import BackupData
from ...models.dumpdata_indent import DumpdataIndent
from ...models.dumpdata_outpuata import DumpdataOutpuata
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    output_file: Union[Unset, str] = UNSET,
    output_data: Union[Unset, DumpdataOutpuata] = UNSET,
    indent: Union[Unset, DumpdataIndent] = UNSET,
) -> Dict[str, Any]:
    url = "{}/dumpdata".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_output_data: Union[Unset, int] = UNSET
    if not isinstance(output_data, Unset):
        json_output_data = output_data.value

    json_indent: Union[Unset, int] = UNSET
    if not isinstance(indent, Unset):
        json_indent = indent.value

    params: Dict[str, Any] = {
        "output-file": output_file,
        "output-data": json_output_data,
        "indent": json_indent,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Union[ApiResponse, BackupData], None, None, None, None]]:
    if response.status_code == 200:

        def _parse_response_200(data: object) -> Union[ApiResponse, BackupData]:
            try:
                response_200_type0: ApiResponse
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type0 = ApiResponse.from_dict(data)

                return response_200_type0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_200_type1: BackupData
            response_200_type1 = BackupData.from_dict(data)

            return response_200_type1

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = None

        return response_400
    if response.status_code == 401:
        response_401 = None

        return response_401
    if response.status_code == 403:
        response_403 = None

        return response_403
    if response.status_code == 500:
        response_500 = None

        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Union[ApiResponse, BackupData], None, None, None, None]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    output_file: Union[Unset, str] = UNSET,
    output_data: Union[Unset, DumpdataOutpuata] = UNSET,
    indent: Union[Unset, DumpdataIndent] = UNSET,
) -> Response[Union[Union[ApiResponse, BackupData], None, None, None, None]]:
    kwargs = _get_kwargs(
        client=client,
        output_file=output_file,
        output_data=output_data,
        indent=indent,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    output_file: Union[Unset, str] = UNSET,
    output_data: Union[Unset, DumpdataOutpuata] = UNSET,
    indent: Union[Unset, DumpdataIndent] = UNSET,
) -> Optional[Union[Union[ApiResponse, BackupData], None, None, None, None]]:
    """ Backups data as data provider independent JSON. The backup can be saved in a local file on the server, to avoid exposing sensitive data over the network, or returned as response body. The output of dumpdata can be used as input for loaddata """

    return sync_detailed(
        client=client,
        output_file=output_file,
        output_data=output_data,
        indent=indent,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    output_file: Union[Unset, str] = UNSET,
    output_data: Union[Unset, DumpdataOutpuata] = UNSET,
    indent: Union[Unset, DumpdataIndent] = UNSET,
) -> Response[Union[Union[ApiResponse, BackupData], None, None, None, None]]:
    kwargs = _get_kwargs(
        client=client,
        output_file=output_file,
        output_data=output_data,
        indent=indent,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    output_file: Union[Unset, str] = UNSET,
    output_data: Union[Unset, DumpdataOutpuata] = UNSET,
    indent: Union[Unset, DumpdataIndent] = UNSET,
) -> Optional[Union[Union[ApiResponse, BackupData], None, None, None, None]]:
    """ Backups data as data provider independent JSON. The backup can be saved in a local file on the server, to avoid exposing sensitive data over the network, or returned as response body. The output of dumpdata can be used as input for loaddata """

    return (
        await asyncio_detailed(
            client=client,
            output_file=output_file,
            output_data=output_data,
            indent=indent,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.admin_filters import AdminFilters
from ..models.admin_permissions import AdminPermissions
from ..models.admin_status import AdminStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Admin")


@attr.s(auto_attribs=True)
class Admin:
    """  """

    id: Union[Unset, int] = UNSET
    status: Union[Unset, AdminStatus] = UNSET
    username: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    permissions: Union[Unset, List[AdminPermissions]] = UNSET
    filters: Union[Unset, AdminFilters] = UNSET
    additional_info: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        status: Union[Unset, int] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        username = self.username
        description = self.description
        password = self.password
        email = self.email
        permissions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = []
            for permissions_item_data in self.permissions:
                permissions_item = permissions_item_data.value

                permissions.append(permissions_item)

        filters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        additional_info = self.additional_info

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if username is not UNSET:
            field_dict["username"] = username
        if description is not UNSET:
            field_dict["description"] = description
        if password is not UNSET:
            field_dict["password"] = password
        if email is not UNSET:
            field_dict["email"] = email
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if filters is not UNSET:
            field_dict["filters"] = filters
        if additional_info is not UNSET:
            field_dict["additional_info"] = additional_info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        status: Union[Unset, AdminStatus] = UNSET
        _status = d.pop("status", UNSET)
        if not isinstance(_status, Unset):
            status = AdminStatus(_status)

        username = d.pop("username", UNSET)

        description = d.pop("description", UNSET)

        password = d.pop("password", UNSET)

        email = d.pop("email", UNSET)

        permissions = []
        _permissions = d.pop("permissions", UNSET)
        for permissions_item_data in _permissions or []:
            permissions_item = AdminPermissions(permissions_item_data)

            permissions.append(permissions_item)

        filters: Union[Unset, AdminFilters] = UNSET
        _filters = d.pop("filters", UNSET)
        if not isinstance(_filters, Unset):
            filters = AdminFilters.from_dict(_filters)

        additional_info = d.pop("additional_info", UNSET)

        admin = cls(
            id=id,
            status=status,
            username=username,
            description=description,
            password=password,
            email=email,
            permissions=permissions,
            filters=filters,
            additional_info=additional_info,
        )

        admin.additional_properties = d
        return admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AdminFilters")


@attr.s(auto_attribs=True)
class AdminFilters:
    """  """

    allow_list: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allow_list: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allow_list, Unset):
            allow_list = self.allow_list

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_list is not UNSET:
            field_dict["allow_list"] = allow_list

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_list = cast(List[str], d.pop("allow_list", UNSET))

        admin_filters = cls(
            allow_list=allow_list,
        )

        admin_filters.additional_properties = d
        return admin_filters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.filesystem_config import FilesystemConfig
from ..models.user_filters import UserFilters
from ..models.user_permissions import UserPermissions
from ..models.user_status import UserStatus
from ..models.virtual_folder import VirtualFolder
from ..types import UNSET, Unset

T = TypeVar("T", bound="User")


@attr.s(auto_attribs=True)
class User:
    """  """

    id: Union[Unset, int] = UNSET
    status: Union[Unset, UserStatus] = UNSET
    username: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    expiration_date: Union[Unset, int] = UNSET
    password: Union[Unset, str] = UNSET
    public_keys: Union[Unset, List[str]] = UNSET
    home_dir: Union[Unset, str] = UNSET
    virtual_folders: Union[Unset, List[VirtualFolder]] = UNSET
    uid: Union[Unset, int] = UNSET
    gid: Union[Unset, int] = UNSET
    max_sessions: Union[Unset, int] = UNSET
    quota_size: Union[Unset, int] = UNSET
    quota_files: Union[Unset, int] = UNSET
    permissions: Union[Unset, UserPermissions] = UNSET
    used_quota_size: Union[Unset, int] = UNSET
    used_quota_files: Union[Unset, int] = UNSET
    last_quota_update: Union[Unset, int] = UNSET
    upload_bandwidth: Union[Unset, int] = UNSET
    download_bandwidth: Union[Unset, int] = UNSET
    last_login: Union[Unset, int] = UNSET
    filters: Union[Unset, UserFilters] = UNSET
    filesystem: Union[Unset, FilesystemConfig] = UNSET
    additional_info: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        status: Union[Unset, int] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        username = self.username
        description = self.description
        expiration_date = self.expiration_date
        password = self.password
        public_keys: Union[Unset, List[str]] = UNSET
        if not isinstance(self.public_keys, Unset):
            public_keys = self.public_keys

        home_dir = self.home_dir
        virtual_folders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.virtual_folders, Unset):
            virtual_folders = []
            for virtual_folders_item_data in self.virtual_folders:
                virtual_folders_item = virtual_folders_item_data.to_dict()

                virtual_folders.append(virtual_folders_item)

        uid = self.uid
        gid = self.gid
        max_sessions = self.max_sessions
        quota_size = self.quota_size
        quota_files = self.quota_files
        permissions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions.to_dict()

        used_quota_size = self.used_quota_size
        used_quota_files = self.used_quota_files
        last_quota_update = self.last_quota_update
        upload_bandwidth = self.upload_bandwidth
        download_bandwidth = self.download_bandwidth
        last_login = self.last_login
        filters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filters, Unset):
            filters = self.filters.to_dict()

        filesystem: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filesystem, Unset):
            filesystem = self.filesystem.to_dict()

        additional_info = self.additional_info

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if username is not UNSET:
            field_dict["username"] = username
        if description is not UNSET:
            field_dict["description"] = description
        if expiration_date is not UNSET:
            field_dict["expiration_date"] = expiration_date
        if password is not UNSET:
            field_dict["password"] = password
        if public_keys is not UNSET:
            field_dict["public_keys"] = public_keys
        if home_dir is not UNSET:
            field_dict["home_dir"] = home_dir
        if virtual_folders is not UNSET:
            field_dict["virtual_folders"] = virtual_folders
        if uid is not UNSET:
            field_dict["uid"] = uid
        if gid is not UNSET:
            field_dict["gid"] = gid
        if max_sessions is not UNSET:
            field_dict["max_sessions"] = max_sessions
        if quota_size is not UNSET:
            field_dict["quota_size"] = quota_size
        if quota_files is not UNSET:
            field_dict["quota_files"] = quota_files
        if permissions is not UNSET:
            field_dict["permissions"] = permissions
        if used_quota_size is not UNSET:
            field_dict["used_quota_size"] = used_quota_size
        if used_quota_files is not UNSET:
            field_dict["used_quota_files"] = used_quota_files
        if last_quota_update is not UNSET:
            field_dict["last_quota_update"] = last_quota_update
        if upload_bandwidth is not UNSET:
            field_dict["upload_bandwidth"] = upload_bandwidth
        if download_bandwidth is not UNSET:
            field_dict["download_bandwidth"] = download_bandwidth
        if last_login is not UNSET:
            field_dict["last_login"] = last_login
        if filters is not UNSET:
            field_dict["filters"] = filters
        if filesystem is not UNSET:
            field_dict["filesystem"] = filesystem
        if additional_info is not UNSET:
            field_dict["additional_info"] = additional_info

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        status: Union[Unset, UserStatus] = UNSET
        _status = d.pop("status", UNSET)
        if not isinstance(_status, Unset):
            status = UserStatus(_status)

        username = d.pop("username", UNSET)

        description = d.pop("description", UNSET)

        expiration_date = d.pop("expiration_date", UNSET)

        password = d.pop("password", UNSET)

        public_keys = cast(List[str], d.pop("public_keys", UNSET))

        home_dir = d.pop("home_dir", UNSET)

        virtual_folders = []
        _virtual_folders = d.pop("virtual_folders", UNSET)
        for virtual_folders_item_data in _virtual_folders or []:
            virtual_folders_item = VirtualFolder.from_dict(virtual_folders_item_data)

            virtual_folders.append(virtual_folders_item)

        uid = d.pop("uid", UNSET)

        gid = d.pop("gid", UNSET)

        max_sessions = d.pop("max_sessions", UNSET)

        quota_size = d.pop("quota_size", UNSET)

        quota_files = d.pop("quota_files", UNSET)

        permissions: Union[Unset, UserPermissions] = UNSET
        _permissions = d.pop("permissions", UNSET)
        if not isinstance(_permissions, Unset):
            permissions = UserPermissions.from_dict(_permissions)

        used_quota_size = d.pop("used_quota_size", UNSET)

        used_quota_files = d.pop("used_quota_files", UNSET)

        last_quota_update = d.pop("last_quota_update", UNSET)

        upload_bandwidth = d.pop("upload_bandwidth", UNSET)

        download_bandwidth = d.pop("download_bandwidth", UNSET)

        last_login = d.pop("last_login", UNSET)

        filters: Union[Unset, UserFilters] = UNSET
        _filters = d.pop("filters", UNSET)
        if not isinstance(_filters, Unset):
            filters = UserFilters.from_dict(_filters)

        filesystem: Union[Unset, FilesystemConfig] = UNSET
        _filesystem = d.pop("filesystem", UNSET)
        if not isinstance(_filesystem, Unset):
            filesystem = FilesystemConfig.from_dict(_filesystem)

        additional_info = d.pop("additional_info", UNSET)

        user = cls(
            id=id,
            status=status,
            username=username,
            description=description,
            expiration_date=expiration_date,
            password=password,
            public_keys=public_keys,
            home_dir=home_dir,
            virtual_folders=virtual_folders,
            uid=uid,
            gid=gid,
            max_sessions=max_sessions,
            quota_size=quota_size,
            quota_files=quota_files,
            permissions=permissions,
            used_quota_size=used_quota_size,
            used_quota_files=used_quota_files,
            last_quota_update=last_quota_update,
            upload_bandwidth=upload_bandwidth,
            download_bandwidth=download_bandwidth,
            last_login=last_login,
            filters=filters,
            filesystem=filesystem,
            additional_info=additional_info,
        )

        user.additional_properties = d
        return user

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.extensions_filter import ExtensionsFilter
from ..models.login_methods import LoginMethods
from ..models.patterns_filter import PatternsFilter
from ..models.supported_protocols import SupportedProtocols
from ..models.user_filters_tls_username import UserFiltersTlsUsername
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserFilters")


@attr.s(auto_attribs=True)
class UserFilters:
    """ Additional user restrictions """

    allowed_ip: Union[Unset, List[str]] = UNSET
    denied_ip: Union[Unset, List[str]] = UNSET
    denied_login_methods: Union[Unset, List[LoginMethods]] = UNSET
    denied_protocols: Union[Unset, List[SupportedProtocols]] = UNSET
    file_patterns: Union[Unset, List[PatternsFilter]] = UNSET
    file_extensions: Union[Unset, List[ExtensionsFilter]] = UNSET
    max_upload_file_size: Union[Unset, int] = UNSET
    tls_username: Union[Unset, UserFiltersTlsUsername] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allowed_ip: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_ip, Unset):
            allowed_ip = self.allowed_ip

        denied_ip: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_ip, Unset):
            denied_ip = self.denied_ip

        denied_login_methods: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_login_methods, Unset):
            denied_login_methods = []
            for denied_login_methods_item_data in self.denied_login_methods:
                denied_login_methods_item = denied_login_methods_item_data.value

                denied_login_methods.append(denied_login_methods_item)

        denied_protocols: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_protocols, Unset):
            denied_protocols = []
            for denied_protocols_item_data in self.denied_protocols:
                denied_protocols_item = denied_protocols_item_data.value

                denied_protocols.append(denied_protocols_item)

        file_patterns: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.file_patterns, Unset):
            file_patterns = []
            for file_patterns_item_data in self.file_patterns:
                file_patterns_item = file_patterns_item_data.to_dict()

                file_patterns.append(file_patterns_item)

        file_extensions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.file_extensions, Unset):
            file_extensions = []
            for file_extensions_item_data in self.file_extensions:
                file_extensions_item = file_extensions_item_data.to_dict()

                file_extensions.append(file_extensions_item)

        max_upload_file_size = self.max_upload_file_size
        tls_username: Union[Unset, str] = UNSET
        if not isinstance(self.tls_username, Unset):
            tls_username = self.tls_username.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allowed_ip is not UNSET:
            field_dict["allowed_ip"] = allowed_ip
        if denied_ip is not UNSET:
            field_dict["denied_ip"] = denied_ip
        if denied_login_methods is not UNSET:
            field_dict["denied_login_methods"] = denied_login_methods
        if denied_protocols is not UNSET:
            field_dict["denied_protocols"] = denied_protocols
        if file_patterns is not UNSET:
            field_dict["file_patterns"] = file_patterns
        if file_extensions is not UNSET:
            field_dict["file_extensions"] = file_extensions
        if max_upload_file_size is not UNSET:
            field_dict["max_upload_file_size"] = max_upload_file_size
        if tls_username is not UNSET:
            field_dict["tls_username"] = tls_username

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allowed_ip = cast(List[str], d.pop("allowed_ip", UNSET))

        denied_ip = cast(List[str], d.pop("denied_ip", UNSET))

        denied_login_methods = []
        _denied_login_methods = d.pop("denied_login_methods", UNSET)
        for denied_login_methods_item_data in _denied_login_methods or []:
            denied_login_methods_item = LoginMethods(denied_login_methods_item_data)

            denied_login_methods.append(denied_login_methods_item)

        denied_protocols = []
        _denied_protocols = d.pop("denied_protocols", UNSET)
        for denied_protocols_item_data in _denied_protocols or []:
            denied_protocols_item = SupportedProtocols(denied_protocols_item_data)

            denied_protocols.append(denied_protocols_item)

        file_patterns = []
        _file_patterns = d.pop("file_patterns", UNSET)
        for file_patterns_item_data in _file_patterns or []:
            file_patterns_item = PatternsFilter.from_dict(file_patterns_item_data)

            file_patterns.append(file_patterns_item)

        file_extensions = []
        _file_extensions = d.pop("file_extensions", UNSET)
        for file_extensions_item_data in _file_extensions or []:
            file_extensions_item = ExtensionsFilter.from_dict(file_extensions_item_data)

            file_extensions.append(file_extensions_item)

        max_upload_file_size = d.pop("max_upload_file_size", UNSET)

        tls_username: Union[Unset, UserFiltersTlsUsername] = UNSET
        _tls_username = d.pop("tls_username", UNSET)
        if not isinstance(_tls_username, Unset):
            tls_username = UserFiltersTlsUsername(_tls_username)

        user_filters = cls(
            allowed_ip=allowed_ip,
            denied_ip=denied_ip,
            denied_login_methods=denied_login_methods,
            denied_protocols=denied_protocols,
            file_patterns=file_patterns,
            file_extensions=file_extensions,
            max_upload_file_size=max_upload_file_size,
            tls_username=tls_username,
        )

        user_filters.additional_properties = d
        return user_filters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .admin import Admin
from .admin_filters import AdminFilters
from .admin_permissions import AdminPermissions
from .admin_status import AdminStatus
from .api_response import ApiResponse
from .azure_blob_fs_config import AzureBlobFsConfig
from .azure_blob_fs_config_access_tier import AzureBlobFsConfigAccessTier
from .backup_data import BackupData
from .ban_status import BanStatus
from .base_virtual_folder import BaseVirtualFolder
from .connection_status import ConnectionStatus
from .connection_status_protocol import ConnectionStatusProtocol
from .crypt_fs_config import CryptFsConfig
from .data_provider_status import DataProviderStatus
from .dir_permissions import DirPermissions
from .dumpdata_indent import DumpdataIndent
from .dumpdata_outpuata import DumpdataOutpuata
from .extensions_filter import ExtensionsFilter
from .filesystem_config import FilesystemConfig
from .filesystem_config_provider import FilesystemConfigProvider
from .folder_quota_scan import FolderQuotaScan
from .folder_quota_update_mode import FolderQuotaUpdateMode
from .ftp_passive_port_range import FTPPassivePortRange
from .ftp_service_status import FTPServiceStatus
from .ftpd_binding import FTPDBinding
from .ftpd_binding_tls_mode import FTPDBindingTlsMode
from .gcs_config import GCSConfig
from .gcs_config_automatic_credentials import GCSConfigAutomaticCredentials
from .get_admins_order import GetAdminsOrder
from .get_folders_order import GetFoldersOrder
from .get_users_order import GetUsersOrder
from .login_methods import LoginMethods
from .patterns_filter import PatternsFilter
from .permission import Permission
from .pwd_change import PwdChange
from .quota_scan import QuotaScan
from .quota_update_mode import QuotaUpdateMode
from .s3_config import S3Config
from .score_status import ScoreStatus
from .secret import Secret
from .secret_status import SecretStatus
from .services_status import ServicesStatus
from .services_status_defender import ServicesStatusDefender
from .sftp_fs_config import SFTPFsConfig
from .ssh_binding import SSHBinding
from .ssh_host_key import SSHHostKey
from .ssh_service_status import SSHServiceStatus
from .supported_protocols import SupportedProtocols
from .token import Token
from .transfer import Transfer
from .transfer_operation_type import TransferOperationType
from .unban_host_json_body import UnbanHostJsonBody
from .update_user_disconnect import UpdateUserDisconnect
from .user import User
from .user_filters import UserFilters
from .user_filters_tls_username import UserFiltersTlsUsername
from .user_permissions import UserPermissions
from .user_status import UserStatus
from .version_info import VersionInfo
from .virtual_folder import VirtualFolder
from .web_dav_binding import WebDAVBinding
from .web_dav_service_status import WebDAVServiceStatus



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ExtensionsFilter")


@attr.s(auto_attribs=True)
class ExtensionsFilter:
    """  """

    path: Union[Unset, str] = UNSET
    allowed_extensions: Union[Unset, List[str]] = UNSET
    denied_extensions: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        allowed_extensions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_extensions, Unset):
            allowed_extensions = self.allowed_extensions

        denied_extensions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_extensions, Unset):
            denied_extensions = self.denied_extensions

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if allowed_extensions is not UNSET:
            field_dict["allowed_extensions"] = allowed_extensions
        if denied_extensions is not UNSET:
            field_dict["denied_extensions"] = denied_extensions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("path", UNSET)

        allowed_extensions = cast(List[str], d.pop("allowed_extensions", UNSET))

        denied_extensions = cast(List[str], d.pop("denied_extensions", UNSET))

        extensions_filter = cls(
            path=path,
            allowed_extensions=allowed_extensions,
            denied_extensions=denied_extensions,
        )

        extensions_filter.additional_properties = d
        return extensions_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ConnectionStatusProtocol(str, Enum):
    SFTP = "SFTP"
    SCP = "SCP"
    SSH = "SSH"
    FTP = "FTP"
    DAV = "DAV"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.transfer_operation_type import TransferOperationType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Transfer")


@attr.s(auto_attribs=True)
class Transfer:
    """  """

    operation_type: Union[Unset, TransferOperationType] = UNSET
    path: Union[Unset, str] = UNSET
    start_time: Union[Unset, int] = UNSET
    size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        operation_type: Union[Unset, str] = UNSET
        if not isinstance(self.operation_type, Unset):
            operation_type = self.operation_type.value

        path = self.path
        start_time = self.start_time
        size = self.size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if operation_type is not UNSET:
            field_dict["operation_type"] = operation_type
        if path is not UNSET:
            field_dict["path"] = path
        if start_time is not UNSET:
            field_dict["start_time"] = start_time
        if size is not UNSET:
            field_dict["size"] = size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        operation_type: Union[Unset, TransferOperationType] = UNSET
        _operation_type = d.pop("operation_type", UNSET)
        if not isinstance(_operation_type, Unset):
            operation_type = TransferOperationType(_operation_type)

        path = d.pop("path", UNSET)

        start_time = d.pop("start_time", UNSET)

        size = d.pop("size", UNSET)

        transfer = cls(
            operation_type=operation_type,
            path=path,
            start_time=start_time,
            size=size,
        )

        transfer.additional_properties = d
        return transfer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class Permission(str, Enum):
    VALUE_0 = "*"
    LIST = "list"
    DOWNLOAD = "download"
    UPLOAD = "upload"
    OVERWRITE = "overwrite"
    DELETE = "delete"
    RENAME = "rename"
    CREATE_DIRS = "create_dirs"
    CREATE_SYMLINKS = "create_symlinks"
    CHMOD = "chmod"
    CHOWN = "chown"
    CHTIMES = "chtimes"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.admin import Admin
from ..models.base_virtual_folder import BaseVirtualFolder
from ..models.user import User
from ..types import UNSET, Unset

T = TypeVar("T", bound="BackupData")


@attr.s(auto_attribs=True)
class BackupData:
    """  """

    users: Union[Unset, List[User]] = UNSET
    folders: Union[Unset, List[BaseVirtualFolder]] = UNSET
    admins: Union[Unset, List[Admin]] = UNSET
    version: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        users: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.users, Unset):
            users = []
            for users_item_data in self.users:
                users_item = users_item_data.to_dict()

                users.append(users_item)

        folders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.folders, Unset):
            folders = []
            for folders_item_data in self.folders:
                folders_item = folders_item_data.to_dict()

                folders.append(folders_item)

        admins: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.admins, Unset):
            admins = []
            for admins_item_data in self.admins:
                admins_item = admins_item_data.to_dict()

                admins.append(admins_item)

        version = self.version

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if users is not UNSET:
            field_dict["users"] = users
        if folders is not UNSET:
            field_dict["folders"] = folders
        if admins is not UNSET:
            field_dict["admins"] = admins
        if version is not UNSET:
            field_dict["version"] = version

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        users = []
        _users = d.pop("users", UNSET)
        for users_item_data in _users or []:
            users_item = User.from_dict(users_item_data)

            users.append(users_item)

        folders = []
        _folders = d.pop("folders", UNSET)
        for folders_item_data in _folders or []:
            folders_item = BaseVirtualFolder.from_dict(folders_item_data)

            folders.append(folders_item)

        admins = []
        _admins = d.pop("admins", UNSET)
        for admins_item_data in _admins or []:
            admins_item = Admin.from_dict(admins_item_data)

            admins.append(admins_item)

        version = d.pop("version", UNSET)

        backup_data = cls(
            users=users,
            folders=folders,
            admins=admins,
            version=version,
        )

        backup_data.additional_properties = d
        return backup_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="CryptFsConfig")


@attr.s(auto_attribs=True)
class CryptFsConfig:
    """ Crypt filesystem configuration details """

    passphrase: Union[Unset, Secret] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        passphrase: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.passphrase, Unset):
            passphrase = self.passphrase.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if passphrase is not UNSET:
            field_dict["passphrase"] = passphrase

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        passphrase: Union[Unset, Secret] = UNSET
        _passphrase = d.pop("passphrase", UNSET)
        if not isinstance(_passphrase, Unset):
            passphrase = Secret.from_dict(_passphrase)

        crypt_fs_config = cls(
            passphrase=passphrase,
        )

        crypt_fs_config.additional_properties = d
        return crypt_fs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
